//=========================================================
//  MusE
//  Linux Music Editor
//    $Id: dummyaudio.cpp,v 1.3 2004/02/01 10:41:55 wschweer Exp $
//  (C) Copyright 2002-2003 Werner Schweer (ws@seh.de)
//=========================================================

#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <pthread.h>

#include "config.h"
#include "audio.h"
#include "audiodev.h"
#include "globals.h"
#include "song.h"

//---------------------------------------------------------
//   DummyAudioDevice
//---------------------------------------------------------

class DummyAudioDevice : public AudioDevice {
      pthread_t dummyThread;
      float buffer[1024];
      int state;

   public:
      DummyAudioDevice() {
            state = Audio::STOP;
            }
      virtual ~DummyAudioDevice() {}

      virtual void start();
      virtual void stop ();
      virtual int framePos() const { return 0; }

      virtual float* getBuffer(void* /*port*/, unsigned long nframes)
            {
            if (nframes > 1024) {
                  printf("error: segment size > 1024\n");
                  exit(-1);
                  }
            return buffer;
            }

      virtual std::list<QString> outputPorts();
      virtual std::list<QString> inputPorts();

      virtual void registerClient() {}

      virtual void* registerOutPort(const char*) {
            return (void*)1;
            }
      virtual void* registerInPort(const char*) {
            return (void*)2;
            }
      virtual void unregisterPort(void*) {}
      virtual void connect(void*, void*) {}
      virtual void disconnect(void*, void*) {}
      virtual void setPortName(void*, const char*) {}
      virtual void* findPort(const char*) { return 0;}
      virtual QString portName(void*) {
            return QString("mops");
            }
      virtual int getState() { return state; }
      virtual unsigned getCurFrame() { return 0; }
      virtual bool isRealtime() { return false; }
      virtual void startTransport() {
            state = Audio::PLAY;
            }
      virtual void stopTransport() {
            state = Audio::STOP;
            }
      virtual void seekTransport(unsigned) {
            }
      virtual void setFreewheel(bool) {}
      };

DummyAudioDevice* dummyAudio;

//---------------------------------------------------------
//   initDummyAudio
//---------------------------------------------------------

bool initDummyAudio()
      {
      dummyAudio = new DummyAudioDevice();
      audioDevice = dummyAudio;
      return false;
      }

//---------------------------------------------------------
//   outputPorts
//---------------------------------------------------------

std::list<QString> DummyAudioDevice::outputPorts()
      {
      std::list<QString> clientList;
      clientList.push_back(QString("output1"));
      clientList.push_back(QString("output2"));
      return clientList;
      }

//---------------------------------------------------------
//   inputPorts
//---------------------------------------------------------

std::list<QString> DummyAudioDevice::inputPorts()
      {
      std::list<QString> clientList;
      clientList.push_back(QString("input1"));
      clientList.push_back(QString("input2"));
      return clientList;
      }

//---------------------------------------------------------
//   dummyLoop
//---------------------------------------------------------

static void* dummyLoop(void*)
      {
      for (;;) {
            audio->process(segmentSize);
            usleep(1000*100);
            }
      pthread_exit(0);
      }

void DummyAudioDevice::start()
      {
      pthread_attr_t* attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
      pthread_attr_init(attributes);
      if (pthread_create(&dummyThread, attributes, ::dummyLoop, this))
            perror("creating thread failed:");
      pthread_attr_destroy(attributes);
      }

void DummyAudioDevice::stop ()
      {
      pthread_cancel(dummyThread);
      pthread_join(dummyThread, 0);
      }

